@@ -124,7 +124,7 @@ static int fs_proc_check(const char *fs_name)
 
 	f = fopen("/proc/filesystems", "r");
 	if (!f)
-		return (0);
+		return 0;
 	while (fgets(buf, sizeof(buf), f)) {
 		cp = buf;
 		if (!isspace(*cp)) {
@@ -133,19 +133,22 @@ static int fs_proc_check(const char *fs_name)
 		}
 		while (*cp && isspace(*cp))
 			cp++;
-		if ((t = strchr(cp, '\n')) != NULL)
+		t = strchr(cp, '\n');
+		if (t != NULL)
 			*t = 0;
-		if ((t = strchr(cp, '\t')) != NULL)
+		t = strchr(cp, '\t');
+		if (t != NULL)
 			*t = 0;
-		if ((t = strchr(cp, ' ')) != NULL)
+		t = strchr(cp, ' ');
+		if (t != NULL)
 			*t = 0;
 		if (!strcmp(fs_name, cp)) {
 			fclose(f);
-			return (1);
+			return 1;
 		}
 	}
 	fclose(f);
-	return (0);
+	return 0;
 }
 
 /*
@@ -160,18 +163,20 @@ static int check_for_modules(const char *fs_name)
 	int		i;
 
 	if (uname(&uts))
-		return (0);
+		return 0;
 	snprintf(buf, sizeof(buf), "/lib/modules/%s/modules.dep", uts.release);
 
 	f = fopen(buf, "r");
 	if (!f)
-		return (0);
+		return 0;
 	while (fgets(buf, sizeof(buf), f)) {
-		if ((cp = strchr(buf, ':')) != NULL)
+		cp = strchr(buf, ':');
+		if (cp != NULL)
 			*cp = 0;
 		else
 			continue;
-		if ((cp = strrchr(buf, '/')) != NULL)
+		cp = strrchr(buf, '/');
+		if (cp != NULL)
 			cp++;
 		i = strlen(cp);
 		if (i > 3) {
@@ -181,11 +186,11 @@ static int check_for_modules(const char *fs_name)
 		}
 		if (!strcmp(cp, fs_name)) {
 			fclose(f);
-			return (1);
+			return 1;
 		}
 	}
 	fclose(f);
-	return (0);
+	return 0;
 }
 
 static int base_ext4_image(const void *buf, unsigned long long *bytes,
@@ -318,7 +323,8 @@ static int jfs_image(const void *buf, unsigned long long *bytes)
 	const struct jfs_superblock *sb = (const struct jfs_superblock *)buf;
 
 	if (!memcmp(sb->s_magic, JFS_MAGIC, 4)) {
-		*bytes = __le64_to_cpu(sb->s_size) << __le16_to_cpu(sb->s_l2pbsize);
+		*bytes = __le64_to_cpu(sb->s_size)
+			<< __le16_to_cpu(sb->s_l2pbsize);
 		return 1;
 	}
 	return 0;
@@ -516,7 +522,7 @@ int identify_fs(int fd, const char **fstype,
 		unsigned long long *bytes, off_t offset)
 {
 	uint64_t buf[BLOCK_SIZE >> 3];	/* 64-bit worst case alignment */
-	off_t cur_block = (off_t) - 1;
+	off_t cur_block = (off_t) -1;
 	struct imagetype *ip;
 	int ret;
 	unsigned long long dummy;
