@@ -1,4 +1,3 @@
-#include <errno.h>
 #include <poll.h>
 #include <limits.h>
 #include <setjmp.h>
@@ -85,7 +84,8 @@ static void configure_device(struct netdev *dev)
 		       dev->name, dev->mtu);
 
 	if (netdev_setaddress(dev))
-		printf("IP-Config: failed to set addresses on %s\n", dev->name);
+		printf("IP-Config: failed to set addresses on %s\n",
+		       dev->name);
 	if (netdev_setdefaultroute(dev))
 		printf("IP-Config: failed to set default route on %s\n",
 		       dev->name);
@@ -95,6 +95,30 @@ static void configure_device(struct netdev *dev)
 			dev->hostname, dev->name);
 }
 
+/*
+ * Escape shell varialbes in git style:
+ * Always start with a single quote ('), then leave all characters
+ * except ' and ! unchanged.
+ */
+static void write_option(FILE *f, const char *name, const char *chr)
+{
+
+	fprintf(f, "%s='", name);
+	while (*chr) {
+		switch (*chr) {
+		case '!':
+		case '\'':
+			fprintf(f, "'\\%c'", *chr);
+			break;
+		default:
+			fprintf(f, "%c", *chr);
+			break;
+		}
+		++chr;
+	}
+	fprintf(f, "'\n");
+}
+
 static void dump_device_config(struct netdev *dev)
 {
 	char fn[40];
@@ -103,22 +127,26 @@ static void dump_device_config(struct netdev *dev)
 	snprintf(fn, sizeof(fn), "/tmp/net-%s.conf", dev->name);
 	f = fopen(fn, "w");
 	if (f) {
-		fprintf(f, "DEVICE=%s\n", dev->name);
-		fprintf(f, "IPV4ADDR=%s\n", my_inet_ntoa(dev->ip_addr));
-		fprintf(f, "IPV4BROADCAST=%s\n",
-			my_inet_ntoa(dev->ip_broadcast));
-		fprintf(f, "IPV4NETMASK=%s\n", my_inet_ntoa(dev->ip_netmask));
-		fprintf(f, "IPV4GATEWAY=%s\n", my_inet_ntoa(dev->ip_gateway));
-		fprintf(f, "IPV4DNS0=%s\n",
-			my_inet_ntoa(dev->ip_nameserver[0]));
-		fprintf(f, "IPV4DNS1=%s\n",
-			my_inet_ntoa(dev->ip_nameserver[1]));
-		fprintf(f, "HOSTNAME=%s\n", dev->hostname);
-		fprintf(f, "DNSDOMAIN=%s\n", dev->dnsdomainname);
-		fprintf(f, "NISDOMAIN=%s\n", dev->nisdomainname);
-		fprintf(f, "ROOTSERVER=%s\n", my_inet_ntoa(dev->ip_server));
-		fprintf(f, "ROOTPATH=%s\n", dev->bootpath);
-		fprintf(f, "filename=\"%s\"\n", dev->filename);
+		write_option(f, "DEVICE", dev->name);
+		write_option(f, "IPV4ADDR",
+				my_inet_ntoa(dev->ip_addr));
+		write_option(f, "IPV4BROADCAST",
+				my_inet_ntoa(dev->ip_broadcast));
+		write_option(f, "IPV4NETMASK",
+				my_inet_ntoa(dev->ip_netmask));
+		write_option(f, "IPV4GATEWAY",
+				my_inet_ntoa(dev->ip_gateway));
+		write_option(f, "IPV4DNS0",
+				my_inet_ntoa(dev->ip_nameserver[0]));
+		write_option(f, "IPV4DNS1",
+				my_inet_ntoa(dev->ip_nameserver[1]));
+		write_option(f, "HOSTNAME",  dev->hostname);
+		write_option(f, "DNSDOMAIN", dev->dnsdomainname);
+		write_option(f, "NISDOMAIN", dev->nisdomainname);
+		write_option(f, "ROOTSERVER",
+				my_inet_ntoa(dev->ip_server));
+		write_option(f, "ROOTPATH", dev->bootpath);
+		write_option(f, "filename", dev->filename);
 		fclose(f);
 	}
 }
@@ -148,11 +176,6 @@ static void postprocess_device(struct netdev *dev)
 		printf("IP-Config: %s guessed broadcast address %s\n",
 		       dev->name, my_inet_ntoa(dev->ip_broadcast));
 	}
-	if (dev->ip_nameserver[0] == INADDR_ANY) {
-		dev->ip_nameserver[0] = dev->ip_server;
-		printf("IP-Config: %s guessed nameserver address %s\n",
-		       dev->name, my_inet_ntoa(dev->ip_nameserver[0]));
-	}
 }
 
 static void complete_device(struct netdev *dev)
@@ -168,20 +191,33 @@ static void complete_device(struct netdev *dev)
 	ifaces = dev;
 }
 
+/*
+ * Returns:
+ *  0 = Not handled, try again later
+ *  1 = Handled
+ */
 static int process_receive_event(struct state *s, time_t now)
 {
 	int handled = 1;
 
 	switch (s->state) {
+	case DEVST_ERROR:
+		return 0; /* Not handled */
+	case DEVST_COMPLETE:
+		return 0; /* Not handled as already configured */
+
 	case DEVST_BOOTP:
 		s->restart_state = DEVST_BOOTP;
 		switch (bootp_recv_reply(s->dev)) {
 		case -1:
 			s->state = DEVST_ERROR;
 			break;
+		case 0:
+			handled = 0;
+			break;
 		case 1:
 			s->state = DEVST_COMPLETE;
-			DEBUG(("\n   bootp reply\n"));
+			dprintf("\n   bootp reply\n");
 			break;
 		}
 		break;
@@ -192,7 +228,10 @@ static int process_receive_event(struct state *s, time_t now)
 		case -1:
 			s->state = DEVST_ERROR;
 			break;
-		case 1:	/* Offer received */
+		case 0:
+			handled = 0;
+			break;
+		case DHCPOFFER:	/* Offer received */
 			s->state = DEVST_DHCPREQ;
 			dhcp_send_request(s->dev);
 			break;
@@ -205,14 +244,22 @@ static int process_receive_event(struct state *s, time_t now)
 		case -1:	/* error */
 			s->state = DEVST_ERROR;
 			break;
-		case 1:	/* ACK received */
+		case 0:
+			handled = 0;
+			break;
+		case DHCPACK:	/* ACK received */
 			s->state = DEVST_COMPLETE;
 			break;
-		case 2:	/* NAK received */
+		case DHCPNAK:	/* NAK received */
 			s->state = DEVST_DHCPDISC;
 			break;
 		}
 		break;
+
+	default:
+		dprintf("\n");
+		handled = 0;
+		break;
 	}
 
 	switch (s->state) {
@@ -223,9 +270,6 @@ static int process_receive_event(struct state *s, time_t now)
 	case DEVST_ERROR:
 		/* error occurred, try again in 10 seconds */
 		s->expire = now + 10;
-	default:
-		DEBUG(("\n"));
-		handled = 0;
 		break;
 	}
 
@@ -287,23 +331,18 @@ static void process_timeout_event(struct state *s, time_t now)
 static struct state *slist;
 struct netdev *ifaces;
 
+/*
+ * Returns:
+ *  0 = No dhcp/bootp packet was received
+ *  1 = A packet was received and handled
+ */
 static int do_pkt_recv(int pkt_fd, time_t now)
 {
-	int ifindex, ret;
+	int ret = 0;
 	struct state *s;
 
-	ret = packet_peek(&ifindex);
-	if (ret < 0)
-		goto bail;
-
-	for (s = slist; s; s = s->next) {
-		if (s->dev->ifindex == ifindex) {
-			ret |= process_receive_event(s, now);
-			break;
-		}
-	}
-
-      bail:
+	for (s = slist; s; s = s->next)
+		ret |= process_receive_event(s, now);
 	return ret;
 }
 
@@ -313,7 +352,7 @@ static int loop(void)
 	struct pollfd fds[NR_FDS];
 	struct state *s;
 	int pkt_fd;
-	int nr = 0;
+	int nr = 0, rc = 0;
 	struct timeval now, prev;
 	time_t start;
 
@@ -336,7 +375,7 @@ static int loop(void)
 		int x;
 
 		for (s = slist; s; s = s->next) {
-			DEBUG(("%s: state = %d\n", s->dev->name, s->state));
+			dprintf("%s: state = %d\n", s->dev->name, s->state);
 
 			if (s->state == DEVST_COMPLETE) {
 				done++;
@@ -346,7 +385,7 @@ static int loop(void)
 			pending++;
 
 			if (s->expire - now.tv_sec <= 0) {
-				DEBUG(("timeout\n"));
+				dprintf("timeout\n");
 				process_timeout_event(s, now.tv_sec);
 			}
 
@@ -370,32 +409,30 @@ static int loop(void)
 			gettimeofday(&now, NULL);
 
 			if ((fds[0].revents & POLLRDNORM)) {
-				nr = do_pkt_recv(pkt_fd, now.tv_sec);
-				if (nr == 1)
+				if (do_pkt_recv(pkt_fd, now.tv_sec) == 1)
 					break;
-				else if (nr == 0)
-					packet_discard();
 			}
 
 			if (loop_timeout >= 0 &&
 			    now.tv_sec - start >= loop_timeout) {
 				printf("IP-Config: no response after %d "
 				       "secs - giving up\n", loop_timeout);
+				rc = -1;
 				goto bail;
 			}
 
 			delta_ms = (now.tv_sec - prev.tv_sec) * 1000;
 			delta_ms += (now.tv_usec - prev.tv_usec) / 1000;
 
-			DEBUG(("Delta: %d ms\n", delta_ms));
+			dprintf("Delta: %d ms\n", delta_ms);
 
 			timeout_ms -= delta_ms;
 		}
 	}
-      bail:
+bail:
 	packet_close();
 
-	return 0;
+	return rc;
 }
 
 static int add_one_dev(struct netdev *dev)
@@ -413,13 +450,12 @@ static int add_one_dev(struct netdev *dev)
 	/*
 	 * Select the state that we start from.
 	 */
-	if (dev->caps & CAP_DHCP && dev->ip_addr == INADDR_ANY) {
+	if (dev->caps & CAP_DHCP && dev->ip_addr == INADDR_ANY)
 		state->restart_state = state->state = DEVST_DHCPDISC;
-	} else if (dev->caps & CAP_DHCP) {
+	else if (dev->caps & CAP_DHCP)
 		state->restart_state = state->state = DEVST_DHCPREQ;
-	} else if (dev->caps & CAP_BOOTP) {
+	else if (dev->caps & CAP_BOOTP)
 		state->restart_state = state->state = DEVST_BOOTP;
-	}
 
 	state->next = slist;
 	slist = state;
@@ -427,7 +463,7 @@ static int add_one_dev(struct netdev *dev)
 	return 0;
 }
 
-static void parse_addr(uint32_t * addr, const char *ip)
+static void parse_addr(uint32_t *addr, const char *ip)
 {
 	struct in_addr in;
 	if (inet_aton(ip, &in) == 0) {
@@ -459,7 +495,7 @@ static unsigned int parse_proto(const char *ip)
 		fprintf(stderr, "%s: invalid protocol '%s'\n", progname, ip);
 		longjmp(abort_buf, 1);
 	}
-      bail:
+bail:
 	return caps;
 }
 
@@ -468,10 +504,10 @@ static int add_all_devices(struct netdev *template);
 static int parse_device(struct netdev *dev, const char *ip)
 {
 	char *cp;
-	int i, opt;
+	int opt;
 	int is_ip = 0;
 
-	DEBUG(("IP-Config: parse_device: \"%s\"\n", ip));
+	dprintf("IP-Config: parse_device: \"%s\"\n", ip);
 
 	if (strncmp(ip, "ip=", 3) == 0) {
 		ip += 3;
@@ -482,7 +518,7 @@ static int parse_device(struct netdev *dev, const char *ip)
 	}
 
 	if (!strchr(ip, ':')) {
-		/* Only one option, e.g. "ip=dhcp", or it's an interface name */
+		/* Only one option, e.g. "ip=dhcp", or an interface name */
 		if (is_ip) {
 			dev->caps = parse_proto(ip);
 			bringup_first = 1;
@@ -490,19 +526,20 @@ static int parse_device(struct netdev *dev, const char *ip)
 			dev->name = ip;
 		}
 	} else {
-		for (i = opt = 0; ip && *ip; ip = cp, opt++) {
+		for (opt = 0; ip && *ip; ip = cp, opt++) {
 			if ((cp = strchr(ip, ':'))) {
 				*cp++ = '\0';
 			}
 			if (opt > 6) {
-				fprintf(stderr, "%s: too many options for %s\n",
+				fprintf(stderr,
+					"%s: too many options for %s\n",
 					progname, dev->name);
 				longjmp(abort_buf, 1);
 			}
 
 			if (*ip == '\0')
 				continue;
-			DEBUG(("IP-Config: opt #%d: '%s'\n", opt, ip));
+			dprintf("IP-Config: opt #%d: '%s'\n", opt, ip);
 			switch (opt) {
 			case 0:
 				parse_addr(&dev->ip_addr, ip);
@@ -520,6 +557,8 @@ static int parse_device(struct netdev *dev, const char *ip)
 			case 4:
 				strncpy(dev->hostname, ip, SYS_NMLN - 1);
 				dev->hostname[SYS_NMLN - 1] = '\0';
+				memcpy(dev->reqhostname, dev->hostname,
+				       SYS_NMLN);
 				break;
 			case 5:
 				dev->name = ip;
@@ -543,11 +582,10 @@ static int parse_device(struct netdev *dev, const char *ip)
 static void bringup_device(struct netdev *dev)
 {
 	if (netdev_up(dev) == 0) {
-		if (dev->caps) {
+		if (dev->caps)
 			add_one_dev(dev);
-		} else {
+		else
 			complete_device(dev);
-		}
 	}
 }
 
@@ -567,6 +605,8 @@ static void bringup_one_dev(struct netdev *template, struct netdev *dev)
 		dev->ip_nameserver[1] = template->ip_nameserver[1];
 	if (template->hostname[0] != '\0')
 		strcpy(dev->hostname, template->hostname);
+	if (template->reqhostname[0] != '\0')
+		strcpy(dev->reqhostname, template->reqhostname);
 	dev->caps &= template->caps;
 
 	bringup_device(dev);
@@ -603,7 +643,7 @@ static struct netdev *add_device(const char *info)
 	       dev->caps & CAP_BOOTP ? " BOOTP" : "",
 	       dev->caps & CAP_RARP ? " RARP" : "");
 	return dev;
-      bail:
+bail:
 	free(dev);
 	return NULL;
 }
@@ -622,7 +662,8 @@ static int add_all_devices(struct netdev *template)
 		return 0;
 
 	while ((de = readdir(d)) != NULL) {
-		/* This excludes devices beginning with dots or "dummy", as well as . or .. */
+		/* This excludes devices beginning with dots or "dummy",
+		   as well as . or .. */
 		if (de->d_name[0] == '.' || !strcmp(de->d_name, ".."))
 			continue;
 		i = snprintf(t, PATH_MAX - 1, "%s/%s/flags", sysfs_class_net,
@@ -648,9 +689,10 @@ static int add_all_devices(struct netdev *template)
 		   logic the in-kernel ipconfig uses... */
 		if (!(flags & IFF_LOOPBACK) &&
 		    (flags & (IFF_BROADCAST | IFF_POINTOPOINT))) {
-			DEBUG(("Trying to bring up %s\n", de->d_name));
+			dprintf("Trying to bring up %s\n", de->d_name);
 
-			if (!(dev = add_device(de->d_name)))
+			dev = add_device(de->d_name);
+			if (!dev)
 				continue;
 			bringup_one_dev(template, dev);
 		}
@@ -703,7 +745,7 @@ int ipconfig_main(int argc, char *argv[])
 {
 	struct netdev *dev;
 	int c, port;
-	int err;
+	int err = 0;
 
 	/* If progname is set we're invoked from another program */
 	if (!progname) {
@@ -778,10 +820,11 @@ int ipconfig_main(int argc, char *argv[])
 	if (check_autoconfig()) {
 		if (cfg_local_port != LOCAL_PORT) {
 			printf("IP-Config: binding source port to %d, "
-			       "dest to %d\n", cfg_local_port, cfg_remote_port);
+			       "dest to %d\n",
+			       cfg_local_port, cfg_remote_port);
 		}
-		loop();
+		err = loop();
 	}
 
-	return 0;
+	return err;
 }
